/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "holdem.h"

void holdemMatchInit(holdemmatch_t *match) {
  uint8_t i;
  holdemplayer_t *player;

  // Set Blinds to nil.
  match->blindBig = 0;
  match->blindSmall = 0;

  // Reset the players
  for(i = 0; i < HOLDEM_PLAYER_COUNT; i++) {
    player = match->players + i;

    // Set their state to 0
    player->state = 0x00;

    // Set their chips to zero
    player->chips = 0;
  }
}

void holdemRoundInit(holdemmatch_t *match) {
  uint8_t i;
  holdemplayer_t *player;

  // Refill the deck
  cardDeckFill(match->deck);
  match->deckSize = CARD_DECK_SIZE;

  // Reset the pot
  match->pot = 0;

  // Reset the cards
  match->cardsFacing = 0;
  
  // Reset the players
  for(i = 0; i < HOLDEM_PLAYER_COUNT; i++) {
    player = match->players + i;

    // Clear Round State(s)
    player->state &= ~(
      HOLDEM_STATE_FOLDED | 
      HOLDEM_STATE_SHOWING
    );

    player->cardCount = 0;

    // Set the players' current bet to zero.
    player->currentBet = 0;
  }
}

void holdemDeal(holdemmatch_t *match, holdemplayer_t *player) {
  cardDeal(match->deck, player->cards, match->deckSize, player->cardCount);
  match->deckSize--;
  player->cardCount++;
}

void holdemDealAll(holdemmatch_t *match, uint8_t count) {
  uint8_t i, j;
  for(i = 0; i < count; i++) {
    for(j = 0; j < HOLDEM_PLAYER_COUNT; j++) {
      holdemDeal(match, match->players + j);
    }
  }
}

void holdemFlop(holdemmatch_t *match) {
  if(match->cardsFacing >= HOLDEM_DEALER_HAND) return;
  uint8_t i, count;


  // Burn the card off the top
  match->deckSize -= 1;

  // Change count depending on facing
  count = match->cardsFacing == 0 ? 0x03 : 0x01;

  // Deal
  for(i = 0; i < count; i++) {
    cardDeal(match->deck, match->cards, match->deckSize, match->cardsFacing);
    match->deckSize -= 1;
    match->cardsFacing += 1;
  }
}

uint32_t holdemBet(holdemmatch_t *match, holdemplayer_t *player, uint32_t amount) {
  uint32_t realAmount = mathMin(player->chips, amount);
  match->pot += realAmount;
  player->chips -= realAmount;
  return realAmount;
}